@@ -528,7 +528,8 @@ class FeatureGenerator:
 
         return df
 
-    def compute_cross_sectional_features(self, df_all, rank_method="dense"):
+    def compute_cross_sectional_features(self, df_all, datetime_column_name= "Datetime",
+                                         rank_method="dense"):
         """
         Compute cross-sectional features across multiple tickers using precomputed per-ticker data.
 
@@ -547,18 +548,28 @@ class FeatureGenerator:
             - Cross-sectional ranks for each specified return period.
             - Cross-sectional z-scores for each return period.
         """
+        print(df_all)
+        print(df_all.columns)
 
         zscore_cols = [col for col in df_all.columns if "Z_Score" in col]
+        print(zscore_cols)
         for col in zscore_cols:
             rank_name = f"{col}_Cross_Sectional_Rank"
-            df_all[rank_name] = df_all.groupby(level="Date")[col].rank(method=rank_method, ascending=False)
+            df_all[rank_name] = df_all.groupby(datetime_column_name)[col].rank(method=rank_method,
+                                                                                     ascending=False)
 
         # Step 3: Auto-detect pct_change columns and rank them cross-sectionally
         pct_cols = [col for col in df_all.columns if "pct_change" in col]
+        print(pct_cols)
         for col in pct_cols:
             rank_name = f"{col}_Cross_Sectional_Rank"
-            df_all[rank_name] = df_all.groupby(level="Date")[col].rank(method=rank_method, ascending=False)
-
+            df_all[rank_name] = df_all.groupby(datetime_column_name)[col].rank(method=rank_method,
+                                                                                     ascending=False)
+        spread_cols = [col for col in df_all.columns if "Spread" in col]
+        for col in spread_cols:
+            rank_name = f"{col}_Cross_Sectional_Rank"
+            df_all[rank_name] = df_all.groupby(datetime_column_name)[col].rank(method=rank_method,
+                                                                                     ascending=False)
         return df_all
 
     def compute_sector_spreads(self, data, mapping_df,
@@ -660,8 +671,7 @@ class FeatureGenerator:
         # Sort for rolling ops
         df = df.sort_values([ticker_column_name, datetime_column_name])
         # ---- RETURN SPREADS ----
-        print(df)
-        print(df.columns)
+
         for w in windows:
             # Cumulative returns over horizon w
             df[f"Company_CumRet_{w}D"] = (
@@ -671,6 +681,7 @@ class FeatureGenerator:
                     1 + df.groupby(sector_column_name)["Median_Return"].transform(lambda x: x.rolling(w).sum())
             )
             df[f"Return_Spread_{w}D"] = df[f"Company_CumRet_{w}D"] - df[f"Sector_CumRet_{w}D"]
+            df[f"Return_Spread_{w}D_pct_change"] = df[f"Return_Spread_{w}D"].pct_change()
 
         # ---- VOLATILITY SPREADS ----
         for w in windows:
@@ -681,5 +692,7 @@ class FeatureGenerator:
                 df.groupby(sector_column_name)["Median_Return"].transform(lambda x: x.rolling(w).std())
             )
             df[f"Vol_Spread_{w}D"] = df[f"Company_Vol_{w}D"] - df[f"Sector_Vol_{w}D"]
+            df[f"Vol_Spread_{w}D_pct_change"] = df[f"Vol_Spread_{w}D"].pct_change()
+
 
         return df
